import React, { useEffect, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import {
  scaleLinear,
  scaleBand,
  max,
  brushX,
  select,
} from 'd3';

import { VizContainer } from './VizContainer';
import { useData } from './useData';
import { BarChart } from './BarChart';
import { BarChartZoomed } from './BarChartZoomed';
import { AxisX } from './AxisX';

const width = 960;
const height = 500;
const margin = { top: 40, right: 40, bottom: 80, left: 40 };
const innerWidth = width - margin.right - margin.left;
const innerHeight = height - margin.top - margin.bottom;
const chartHeight = innerHeight / 2;

const App = () => {
  const data = [
    { group: 1, score: 36 },
    { group: 2, score: 40 },
  ];

  const [brushExtent, setBrushExtent] = useState([
    0,
    max(data, (d) => d.score),
  ]);

  const xScale = scaleLinear()
    .domain([0, max(data, (d) => d.score)])
    .range([0, innerWidth]);

  const yScale = scaleBand()
    .domain(data.map((d) => d.group))
    .range([0, chartHeight])
    .paddingInner(0.1)
    .paddingOuter(0.3);

  const xScaleZoom = scaleLinear()
    .domain(brushExtent)
    .range([0, innerWidth]);

  const brushRef = useRef();

  useEffect(() => {
    const brush = brushX().extent([
      [0, 0],
      [innerWidth, innerHeight / 2],
    ]);

    brush.on('brush end', ({ selection }) => {
      setBrushExtent(
        selection
          ? selection.map(xScale.invert)
          : [0, max(data, (d) => d.score)]
      );
    });

    brush(select(brushRef.current));
  }, [innerWidth, innerHeight]);

  return (
    <VizContainer
      width={width}
      height={height}
      margin={margin}
      innerWidth={innerWidth}
      innerHeight={innerHeight}
    >
      <AxisX
        xScale={xScale}
        innerHeight={innerHeight / 2}
      />
      <BarChart
        data={data}
        width={innerWidth}
        height={innerHeight / 2}
        xScale={xScale}
        yScale={yScale}
      />
      <g transform={`translate(0,${innerHeight / 2})`}>
        <AxisX
          xScale={xScaleZoom}
          innerHeight={innerHeight / 2}
        />
        <BarChartZoomed
          data={data}
          width={innerWidth}
          height={innerHeight / 2}
          xScale={xScaleZoom}
          yScale={yScale}
        />
      </g>
      <g ref={brushRef}></g>
    </VizContainer>
  );
};

ReactDOM.render(<App />, document.getElementById('root'));
